
-- Development tools for eihort

------------------------------------------------------------------------------
-- Print all found MC versions to a file
function PrintVersionsList( )
	local file;
	local counter = 1;
	
	local function DoRightSpacing ( version )
		local maxLength = 12;
		local output = "";
		
		for i = string.len(version), maxLength do
			output = output .. " ";
		end
		
		return output .. version;
	end
	
	eihort.createDirectory( Config.deveoper_tools_path );
	file = io.open ( Config.deveoper_tools_path .. "mc versions.txt", "w" );
	
	if file then
		file:write("DETECTED MINECRAFT VERSIONS:\n\nSearched in: " .. string.gsub ( Config.minecraft_path, "\\" , "/" ) .. "versions/*\n\n");
		
		file:write("All detected versions:\n");
		if ProgramState.usedMCVersion then
			while foundVersions[counter] do
				if string.sub( foundVersions[counter][1], 1, 1 ) ~= "." then
					file:write(foundVersions[counter][1]);
					if foundVersions[counter + 1] then
						file:write(", ");
					end
				end
				counter = counter + 1;
			end
			file:write("\n\n");
			
			file:write("Version list with calculation values:\n");
			for _, tab in ipairs ( foundVersions ) do
				if string.sub( tab[1], 1, 1 ) ~= "." then
					file:write(" " .. DoRightSpacing( tab[1] ) .. " = " .. tostring(tab[2]) .. "\n" );
				end
			end
			file:write("\n");
			
			file:write("The best found version is " .. tostring(foundVersions["bestVersion"]) .. " with a value of " .. tostring(foundVersions["bestVersionVal"]) .. "." );
		else
			file:write("eihort was not able to find any version of Minecraft.");
		end
		file:write("\n\n");
		
		file:write("All known version equivalents (unsorted):\n");
		for key, value in pairs ( MinecraftVersions ) do
			file:write(" " .. DoRightSpacing( key ) .. " = " .. value .. "\n" );
		end
		file:write("\n");
		
		file:close();
	end
end

------------------------------------------------------------------------------
-- Print the whole list of loaded assests to a file
function PrintAssetIndex( assetIndex )
	local f;
	
	eihort.createDirectory( Config.deveoper_tools_path );
	
	-- Print assets list to a *.txt file
	f = io.open( Config.deveoper_tools_path .. "assets.txt", "w" );
	if f then
		for k, v in pairs( assetIndex ) do
			f:write( k .. " -> " .. v .. "\n" );
		end
		f:close();
	end
end

------------------------------------------------------------------------------
-- Print the whole list of loaded assests to a file
function PrintCacheHits( hitList )
	local file;
	local cacheQueries, cacheHits = 0, 0;
	
	-- Calculate cache queries and hits
	local function CalculteTotalNumbers ()
		for key, count in pairs( hitList ) do
			cacheQueries = cacheQueries + count
			cacheHits = cacheHits + (count - 1)
		end
	end
	-- Sort cache list most to least hits
	local function SortHitList ()
		local sortedTable = { };
		local checkTable = { };
		
		for path, hits in pairs( hitList ) do			
			if sortedTable[1] then
				for nr, hitTable in ipairs( sortedTable ) do
					if not checkTable[path] then
						if hitList[path] > sortedTable[nr][1] or hitList[path] == sortedTable[nr][1] then
							table.insert ( sortedTable, nr, { hits, path } )
							checkTable[path] = true;
						elseif not sortedTable[nr+1] then
							table.insert ( sortedTable, { hits, path } )
							checkTable[path] = true;
						end
					end
				end
			else
				table.insert ( sortedTable, { hits, path } )
				checkTable[path] = true;
			end
		end
		
		return sortedTable;
	end
	-- Check for the largest hits for one file
	local function FindMostHits( hitList )
		local returnValue = 0;
		
		for nr, hitTable in ipairs( hitList ) do
			if hitList[nr][1] > returnValue then
				returnValue = hitList[nr][1];
			end
		end
		
		return returnValue;
	end
	-- Calculate spaces for the output file
	local function AddSpacer( word, maxLength )
		local output = "";
		
		maxLength = string.len( tostring( maxLength ) );
		
		for	i = string.len( tostring( word ) ), maxLength + 1 do
			output = output .. " ";
		end
		
		return output;
	end
	
	eihort.createDirectory( Config.deveoper_tools_path );
	file = io.open ( Config.deveoper_tools_path .. "cache hits.txt", "w" );
	
	if file then
		file:write("DETECTED CACHE HITS:\n\n");
		
		CalculteTotalNumbers ();
		
		file:write("    Total Quries: " .. cacheQueries .. "\n");
		file:write("      Total Hits: " .. cacheHits .. "\n");
		file:write("  Hits / Queries: " .. string.format( "%.2f", cacheHits/cacheQueries ) .. "\n");
		
		if Config.print_cache_hits_detail then
			local sortedHitList = { };
			local highestNumber, hitsString, counterString;
			
			file:write("\n -- INFORMATION -- \n");
			file:write(" full_tex: texture is located in assets/minecraft/textures/blocks/* and is loaded via blockids.lua\n");
			file:write("  cut_tex: texture is not located in assets/minecraft/textures/blocks/* and is loaded via blocktextures.lua\n");
			
			hitsString = "HITS";	
			sortedHitList = SortHitList();
			highestNumber = FindMostHits( sortedHitList );
			if string.len( hitsString ) > highestNumber then
				highestNumber = string.len( hitsString );
			end
			
			file:write("\n");
			file:write( "     " .. AddSpacer( #sortedHitList, #sortedHitList ) .. "  " .. AddSpacer( hitsString, highestNumber ) .. hitsString .. "  |  TEXTURE\n");
			file:write("-------------------------------------------------------------------------------\n");
			
			for nr, infoTable in ipairs( sortedHitList ) do
				file:write(" " .. AddSpacer( nr * 100, #sortedHitList ) .. "[" .. nr .. "] = " .. AddSpacer( infoTable[1], highestNumber ) .. infoTable[1] .. "  |  " .. infoTable[2] .. "\n");
			end
		end
		file:close();
	end
end

------------------------------------------------------------------------------
-- Developer Buttons

-- Restart eihort button
function RestartEihort( worldPath, eyeX, eyeY, eyeZ, pitch, azimuth, dimemsion )
local executeLine, eihortExtension;

local function VramHandover( )
	local returnValue = "";
	
	if ProgramState.noDetectedVram then
		returnValue = " -vram " .. ProgramState.defaultVRAM;
	end
	
	return returnValue;
end

executeLine = "START \"eihort\" /D";
eihortExtension = "exe"; -- needs to be improved for Linux and Mac

executeLine = executeLine .. " \"" .. Config.eihort_path .. "\" eihort." .. eihortExtension; -- set path
executeLine = executeLine .. " -world \"" .. worldPath .. "\" " .. " -eyeX " .. eyeX .. " -eyeY " .. eyeY .. " -eyeZ " .. eyeZ .. " -pitch " .. pitch .. " -azimuth " .. azimuth .. " -dimemsion " .. dimemsion .. VramHandover(); -- add arguments

os.execute( executeLine ); -- Load new eihort window
QuitFlag = 1; -- Close current eihort window
end
